#!/usr/bin/env python
##################################################
# Gnuradio Python Flow Graph
# Title: Hackrf_Spectrum_Analyzer
# Author: Elektra Wagenrad
# Description: Simple Spectrum Analyzer using Hackrf Source
# Generated: Sun May 31 12:43:19 2015
##################################################

from gnuradio import eng_notation
from gnuradio import gr
from gnuradio import wxgui
from gnuradio.eng_option import eng_option
from gnuradio.fft import window
from gnuradio.filter import firdes
from gnuradio.wxgui import fftsink2
from gnuradio.wxgui import forms
from grc_gnuradio import wxgui as grc_wxgui
from optparse import OptionParser
import osmosdr
import wx

class Hackrf_Spectrum_Analyzer(grc_wxgui.top_block_gui):

    def __init__(self):
        grc_wxgui.top_block_gui.__init__(self, title="Hackrf_Spectrum_Analyzer")
        _icon_path = "/usr/share/icons/hicolor/32x32/apps/gnuradio-grc.png"
        self.SetIcon(wx.Icon(_icon_path, wx.BITMAP_TYPE_ANY))

        ##################################################
        # Variables
        ##################################################
        self.samp_rate = samp_rate = 8000000
        self.rfgain = rfgain = 0
        self.ifgain = ifgain = 8
        self.freq = freq = 100e6
        self.bbgain = bbgain = 10

        ##################################################
        # Blocks
        ##################################################
        self._rfgain_chooser = forms.radio_buttons(
        	parent=self.GetWin(),
        	value=self.rfgain,
        	callback=self.set_rfgain,
        	label="RF-Gain dB",
        	choices=[0,14],
        	labels=[],
        	style=wx.RA_HORIZONTAL,
        )
        self.Add(self._rfgain_chooser)
        self._ifgain_chooser = forms.radio_buttons(
        	parent=self.GetWin(),
        	value=self.ifgain,
        	callback=self.set_ifgain,
        	label="IF-Gain dB",
        	choices=[0,8,16,24,32,40],
        	labels=[],
        	style=wx.RA_HORIZONTAL,
        )
        self.Add(self._ifgain_chooser)
        _freq_sizer = wx.BoxSizer(wx.VERTICAL)
        self._freq_text_box = forms.text_box(
        	parent=self.GetWin(),
        	sizer=_freq_sizer,
        	value=self.freq,
        	callback=self.set_freq,
        	label="Frequency MHz",
        	converter=forms.float_converter(),
        	proportion=0,
        )
        self._freq_slider = forms.slider(
        	parent=self.GetWin(),
        	sizer=_freq_sizer,
        	value=self.freq,
        	callback=self.set_freq,
        	minimum=10e6,
        	maximum=6000e6,
        	num_steps=1000,
        	style=wx.SL_HORIZONTAL,
        	cast=float,
        	proportion=1,
        )
        self.Add(_freq_sizer)
        _bbgain_sizer = wx.BoxSizer(wx.VERTICAL)
        self._bbgain_text_box = forms.text_box(
        	parent=self.GetWin(),
        	sizer=_bbgain_sizer,
        	value=self.bbgain,
        	callback=self.set_bbgain,
        	label="Baseband Gain dB",
        	converter=forms.float_converter(),
        	proportion=0,
        )
        self._bbgain_slider = forms.slider(
        	parent=self.GetWin(),
        	sizer=_bbgain_sizer,
        	value=self.bbgain,
        	callback=self.set_bbgain,
        	minimum=0,
        	maximum=62,
        	num_steps=124,
        	style=wx.SL_HORIZONTAL,
        	cast=float,
        	proportion=1,
        )
        self.Add(_bbgain_sizer)
        self.wxgui_fftsink2_0 = fftsink2.fft_sink_c(
        	self.GetWin(),
        	baseband_freq=freq,
        	y_per_div=5,
        	y_divs=20,
        	ref_level=0,
        	ref_scale=2.0,
        	sample_rate=samp_rate,
        	fft_size=512,
        	fft_rate=10,
        	average=False,
        	avg_alpha=None,
        	title="FFT Plot",
        	peak_hold=False,
        	size=(1200,700),
        )
        self.Add(self.wxgui_fftsink2_0.win)
        self.osmosdr_source_0 = osmosdr.source( args="numchan=" + str(1) + " " + "hackrf=0" )
        self.osmosdr_source_0.set_sample_rate(samp_rate)
        self.osmosdr_source_0.set_center_freq(freq, 0)
        self.osmosdr_source_0.set_freq_corr(0, 0)
        self.osmosdr_source_0.set_dc_offset_mode(0, 0)
        self.osmosdr_source_0.set_iq_balance_mode(0, 0)
        self.osmosdr_source_0.set_gain_mode(False, 0)
        self.osmosdr_source_0.set_gain(rfgain, 0)
        self.osmosdr_source_0.set_if_gain(ifgain, 0)
        self.osmosdr_source_0.set_bb_gain(bbgain, 0)
        self.osmosdr_source_0.set_antenna("", 0)
        self.osmosdr_source_0.set_bandwidth(0, 0)
          

        ##################################################
        # Connections
        ##################################################
        self.connect((self.osmosdr_source_0, 0), (self.wxgui_fftsink2_0, 0))



    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate
        self.osmosdr_source_0.set_sample_rate(self.samp_rate)
        self.wxgui_fftsink2_0.set_sample_rate(self.samp_rate)

    def get_rfgain(self):
        return self.rfgain

    def set_rfgain(self, rfgain):
        self.rfgain = rfgain
        self.osmosdr_source_0.set_gain(self.rfgain, 0)
        self._rfgain_chooser.set_value(self.rfgain)

    def get_ifgain(self):
        return self.ifgain

    def set_ifgain(self, ifgain):
        self.ifgain = ifgain
        self.osmosdr_source_0.set_if_gain(self.ifgain, 0)
        self._ifgain_chooser.set_value(self.ifgain)

    def get_freq(self):
        return self.freq

    def set_freq(self, freq):
        self.freq = freq
        self.osmosdr_source_0.set_center_freq(self.freq, 0)
        self._freq_slider.set_value(self.freq)
        self._freq_text_box.set_value(self.freq)
        self.wxgui_fftsink2_0.set_baseband_freq(self.freq)

    def get_bbgain(self):
        return self.bbgain

    def set_bbgain(self, bbgain):
        self.bbgain = bbgain
        self.osmosdr_source_0.set_bb_gain(self.bbgain, 0)
        self._bbgain_slider.set_value(self.bbgain)
        self._bbgain_text_box.set_value(self.bbgain)

if __name__ == '__main__':
    import ctypes
    import sys
    if sys.platform.startswith('linux'):
        try:
            x11 = ctypes.cdll.LoadLibrary('libX11.so')
            x11.XInitThreads()
        except:
            print "Warning: failed to XInitThreads()"
    parser = OptionParser(option_class=eng_option, usage="%prog: [options]")
    (options, args) = parser.parse_args()
    tb = Hackrf_Spectrum_Analyzer()
    tb.Start(True)
    tb.Wait()
